@@ -59,6 +59,18 @@
 #include <syslog.h>
 #endif
 
+#ifndef OPEN_MAX
+#define OPEN_MAX 20
+#endif
+
+#ifndef WEXITSTATUS
+#define WEXITSTATUS(status) ((unsigned)(status) >> 8)
+#endif
+
+#ifndef _PATH_TTY
+#define _PATH_TTY   "/dev/tty"
+#endif
+
 using namespace UCOMMON_NAMESPACE;
 
 static shell::loglevel_t errlevel = shell::WARN;
@@ -66,6 +78,8 @@ static shell::logmode_t errmode = shell::NONE;
 static const char *errname = NULL;
 static shell::logproc_t errproc = (shell::logproc_t)NULL;
 static mutex_t symlock;
+static shell::downproc_t downproc = (shell::downproc_t)NULL;
+static char **_orig = NULL;
 
 OrderedIndex shell::Option::index;
 const char *shell::domain = NULL;
@@ -574,6 +588,7 @@ mempager(pagesize)
 	_argv0 = NULL;
 	_argv = NULL;
 	_argc = 0;
+	_syms = NULL;
 
 	parse(argc, argv);
 }
@@ -743,6 +758,9 @@ int shell::systemf(const char *format, ...)
 
 void shell::parse(int argc, char **argv)
 {
+	if(!_orig)
+		_orig = argv;
+
 	getargv0(argv);
 	getargv(++argv);
 }
@@ -1322,6 +1340,86 @@ exit:
 	}
 	return pid;
 }
+
+static SERVICE_STATUS_HANDLE hStatus;
+static SERVICE_STATUS status;
+static unsigned detaches = 0;
+
+static void WINAPI control(DWORD control)
+{
+	switch(control) {
+	case 128:
+		if(downproc)
+			(*downproc)(true);
+		return;
+	case SERVICE_CONTROL_SHUTDOWN:
+	case SERVICE_CONTROL_STOP:
+        status.dwCurrentState = SERVICE_STOP_PENDING;
+        status.dwWin32ExitCode = 0;
+        status.dwCheckPoint = 0;
+        status.dwWaitHint = 6000;
+		SetServiceStatus(hStatus, &status);
+		if(downproc)
+			(*downproc)(false);
+		else
+			exit(0);
+		break;
+	default:
+		break;
+	}
+}
+
+extern "C" {
+	extern int main(int argc, char **argv);
+}
+
+static void WINAPI _start(DWORD argc, LPSTR *argv)
+{
+	++detaches;
+	::main(argc, argv);
+}
+
+void shell::detach(downproc_t stop)
+{
+	const char *name = _argv0;
+
+	downproc = stop;
+
+	if(domain)
+		name = domain;
+
+	name = strdup(name);
+
+	if(detaches) {
+		memset(&status, 0, sizeof(SERVICE_STATUS));
+		status.dwServiceType = SERVICE_WIN32;
+		status.dwCurrentState = SERVICE_START_PENDING;
+		status.dwControlsAccepted = SERVICE_ACCEPT_STOP|SERVICE_ACCEPT_SHUTDOWN;
+
+		hStatus = ::RegisterServiceCtrlHandler(name, &control);
+		return;
+	}
+
+	SERVICE_TABLE_ENTRY servicetable[] = {
+		{(LPSTR)name, &_start},
+		{NULL, NULL}
+	};
+
+	if(!StartServiceCtrlDispatcher(servicetable))
+		errexit(1, "*** %s: %s\n", name, _TEXT("failed service start"));
+}
+
+void shell::restart(void)
+{
+}
+
+void shell::up(void)
+{
+	if(detaches) {
+		status.dwCurrentState = SERVICE_RUNNING;
+		::SetServiceStatus(hStatus, &status);
+	}
+}
 	
 int shell::detach(const char *path, char **argv, char **envp, fd_t *stdio)
 {
@@ -1486,6 +1584,97 @@ int shell::system(const char *cmd, const char **envp)
 	::exit(-1);
 }
 
+void shell::up(void)
+{
+}
+
+void shell::restart(void)
+{
+	pid_t pid;
+	int status;
+
+restart:
+	pid = fork();
+	if(pid > 0) {
+		waitpid(pid, &status, 0);
+		if(WIFSIGNALED(status))
+			status = WTERMSIG(status);
+		else
+			status = WIFEXITED(status);
+		switch(status) {
+#ifdef	SIGPWR
+		case SIGPWR:
+#endif
+		case SIGINT:
+		case SIGQUIT:
+		case SIGTERM:
+		case 0:
+			exit(status);
+		default:
+			goto restart;
+		}
+	}
+}
+
+void shell::detach(downproc_t stop)
+{
+	const char *dev = "/dev/null";
+	pid_t pid;
+	int fd;
+
+	downproc = stop;
+
+	close(0);
+	close(1);
+	close(2);
+#ifdef	SIGTTOU
+	signal(SIGTTOU, SIG_IGN);
+#endif
+
+#ifdef	SIGTTIN
+	signal(SIGTTIN, SIG_IGN);
+#endif
+
+#ifdef	SIGTSTP
+	signal(SIGTSTP, SIG_IGN);
+#endif
+	pid = fork();
+	if(pid > 0)
+		exit(0);
+	crit(pid == 0, "detach without process");
+
+#if defined(SIGTSTP) && defined(TIOCNOTTY)
+	crit(setpgid(0, getpid()) == 0, "detach without process group");
+	if((fd = open(_PATH_TTY, O_RDWR)) >= 0) {
+		ioctl(fd, TIOCNOTTY, NULL);
+		close(fd);
+	}
+#else
+
+#ifdef HAVE_SETPGRP
+	crit(setpgrp() == 0, "detach without process group");
+#else
+	crit(setpgid(0, getpid()) == 0, "detach without process group");
+#endif
+	signal(SIGHUP, SIG_IGN);
+	pid = fork();
+	if(pid > 0)
+		exit(0);
+	crit(pid == 0, "detach without process");
+#endif
+	if(dev && *dev) {
+		fd = open(dev, O_RDWR);
+		if(fd > 0)
+			dup2(fd, 0);
+		if(fd != 1)
+			dup2(fd, 1);
+		if(fd != 2)
+			dup2(fd, 2);
+		if(fd > 2)
+			close(fd);
+	}
+}
+
 int shell::detach(const char *path, char **argv, char **envp, fd_t *stdio)
 {
 	char symname[129];
@@ -1913,7 +2102,7 @@ void shell::log(loglevel_t loglevel, const char *fmt, ...)
 		
 #endif
 
-#ifdef	HAVE_EXECVP
+#ifndef _MSWINDOWS_
 
 void shell::restart(char *argv0, char **argv, char **list)
 {
@@ -1934,7 +2123,7 @@ void shell::restart(char *argv0, char **argv, char **list)
 
 #else
 
-void shell::restart(const char *argv0, char **argv, char **list)
+void shell::restart(char *argv0, char **argv, char **list)
 {
 	exit(-1);
 }
