@@ -111,6 +111,11 @@ public:
 	 */
 	typedef bool (*logproc_t)(loglevel_t level, const char *text);
 
+	/**
+	 * Shutdown handler.
+	 */
+	typedef void (*downproc_t)(bool reload);
+
 #ifdef	_MSWINDOWS_
 	typedef	HANDLE pid_t;
 #else
@@ -663,6 +668,11 @@ public:
 	 */
 	void restart(char *argv0, char **argv, char **list);
 
+	/**
+	 * Notify service is up.
+	 */
+	void up(void);
+
 	/**
 	 * Parse shell arguments directly into a shell object.
 	 * @param args table.
@@ -772,6 +782,16 @@ public:
 	inline const char *operator[](unsigned offset)
 		{return _argv[offset];};
 
+	/**
+	 * Detach current process to daemon.
+	 */
+	void detach(downproc_t stopreload = (downproc_t)NULL);
+
+	/**
+	 * Make current process restartable.
+	 */
+	void restart(void);
+
 	/**
 	 * Spawn a child process.  This creates a new child process.  If
 	 * the executable path is a pure filename, then the $PATH will be
@@ -817,7 +837,7 @@ public:
 	 * @param stdio handles for stdin, stdout, and stderr.
 	 * @return 0 if success, -1 on error.
 	 */
-	int	detach(const char *path, char **argv, char **env = NULL, fd_t *stdio = NULL);
+	static int	detach(const char *path, char **argv, char **env = NULL, fd_t *stdio = NULL);
 
 	/**
 	 * Wait for a child process to terminate.  This operation blocks.
